use crate::{objective_function::ObjectiveFun, traits::XVar};

pub enum LineSearch {
    ConstAlpha {
        learning_rate: f64,
    },
    BackTrack {
        max_iterations: usize,
        gamma: f64,
        c: f64,
    },
}

impl LineSearch {
    pub fn get_learning_rate<T, E>(
        &self,
        fun: &dyn ObjectiveFun<T, E>,
        xs: &T,
        direction: &E,
    ) -> f64
    where
        T: XVar<E> + Clone,
        E:,
    {
        match self {
            LineSearch::ConstAlpha { learning_rate } => *learning_rate,
            LineSearch::BackTrack {
                max_iterations,
                gamma,
                c,
            } => {
                let prime = fun.prime(xs);
                let mut del_f = T::scale_prime(&prime, *c);
                let mut new_f = fun.eval(&xs.update(-1.0, &prime));
                let mut t = 1.0;
                for i in 0..*max_iterations {
                    if new_f > T::prime_inner_product(&T::scale_prime(&del_f, t), direction) {
                        break;
                    }
                    t *= gamma;
                    let new_x = xs.update(-t, &prime);
                    new_f = fun.eval(&new_x);
                    del_f = fun.prime(&new_x);
                }
                t
            }
        }
    }
}
